@@ -8,7 +8,7 @@ lmrob <-
 {
     ## to avoid problems with setting argument
     ## call lmrob.control here either with or without method arg.
-    if (missing(control))
+    if (miss.ctrl <- missing(control))
 	control <- if (missing(method))
 	    lmrob.control(...) else lmrob.control(method = method, ...)
     else ## check dots
@@ -33,7 +33,7 @@ lmrob <-
     if(!is.null(offset) && length(offset) != NROW(y))
 	stop(gettextf("number of offsets is %d, should equal %d (number of observations)",
 		      length(offset), NROW(y)), domain = NA)
-    if (!missing(control) && !missing(method) && method != control$method) {
+    if (!miss.ctrl && !missing(method) && method != control$method) {
 	warning("Methods argument set by method is different from method in control\n",
 		"Using the former, method = ", method)
 	control$method <- method
@@ -87,15 +87,25 @@ lmrob <-
 	    y <- wts * y
 	}
 	## check for singular fit
-	## from lm.fit:
-	z0 <- .Call(stats:::C_Cdqrls, x, y, tol = control$solve.tol)
-	singular.fit <- z0$rank < p
-	if (z0$rank > 0) {
+
+        ## faster, but no longer "allowed" by the Crania:
+        ## z0 <- .Call(stats:::C_Cdqrls, x, y, tol = control$solve.tol)
+	if(getRversion() >= "3.1.0") {
+	    z0 <- .lm.fit(x, y, tol = control$solve.tol)
+	    piv <- z0$pivot
+	} else {
+	    z0 <- lm.fit(x, y, tol = control$solve.tol)
+	    piv <- z0$qr$pivot
+	}
+	rankQR <- z0$rank
+
+	singular.fit <- rankQR < p
+	if (rankQR > 0) {
 	    if (singular.fit) {
 		if (!singular.ok) stop("singular fit encountered")
-		pivot <- z0$pivot
-		p1 <- pivot[seq_len(z0$rank)]
-		p2 <- pivot[(z0$rank+1):p]
+		pivot <- piv
+		p1 <- pivot[seq_len(rankQR)]
+		p2 <- pivot[(rankQR+1):p]
 		## to avoid problems in the internal fitting methods,
 		## split into singular and non-singular matrices,
 		## can still re-add singular part later
@@ -103,12 +113,17 @@ lmrob <-
 		x <- x[,p1]
 		attr(x, "assign") <- assign[p1] ## needed for splitFrame to work
 	    }
-	    if (!is.null(init)) {
+	    if (!is.null(ini <- init)) {
 		if (is.character(init)) {
 		    init <- switch(init,
-				   `M-S` = lmrob.M.S(x, y, control, mf),
-				   S = lmrob.S(x, y, control),
+				   "M-S" = lmrob.M.S(x, y, control, mf),
+				   "S"   = lmrob.S  (x, y, control),
 				   stop('init must be "S", "M-S", function or list'))
+		    if(ini == "M-S") { ## "M-S" sometimes reverts to "S":
+			ini <- init$control$method
+                        ## if(identical(ini, "M-S"))
+                        ##     control$method <- paste0(ini, control$method)
+                    }
 		} else if (is.function(init)) {
 		    init <- init(x=x, y=y, control=control, mf=mf)
 		} else if (is.list(init)) {
@@ -124,8 +139,8 @@ lmrob <-
 			    stop("Length of initial coefficients vector does not match rank of singular design matrix x")
 		    }
 		} else stop("unknown init argument")
-		stopifnot(!is.null(init$coef), !is.null(init$scale))
-		## modify (default) control$method
+		stopifnot(is.numeric(init$coef), is.numeric(init$scale))
+		## modify (default) control$method, possibly dropping first letter:
 		if (control$method == "MM" || substr(control$method, 1, 1) == "S")
 		    control$method <- substring(control$method, 2)
 		## check for control$cov argument
@@ -133,6 +148,8 @@ lmrob <-
 		    control$cov <- ".vcov.w"
 	    }
 	    z <- lmrob.fit(x, y, control, init=init) #-> ./lmrob.MM.R
+            if(is.character(ini) && !grepl(paste0("^", ini), control$method))
+                control$method <- paste0(ini, control$method)
 	    if (singular.fit) {
 		coef <- numeric(p)
 		coef[p2] <- NA
@@ -141,14 +158,15 @@ lmrob <-
 		z$coefficients <- coef
 		## Update QR decomposition (z$qr)
 		## pad qr and qraux with zeroes (columns that were pivoted to the right in z0)
-		qr0 <- matrix(0, length(y), p)
-		qr0[,1L:z0$rank] <- z$qr$qr
-		rownames(qr0) <- dn[[1L]]
-		colnames(qr0) <- dn[[2L]][z0$pivot]
-		z$qr$qr <- qr0
-		z$qr$qraux <- c(z$qr$qraux, rep.int(0, p-z0$rank))
-		## set pivot
-		z$qr$pivot <- z0$pivot
+                d.p <- p-rankQR
+                n <- NROW(y)
+		z$qr[c("qr","qraux","pivot")] <-
+		    list(matrix(c(z$qr$qr, rep.int(0, d.p*n)), n, p,
+				dimnames = list(dn[[1L]], dn[[2L]][piv])),
+			 ## qraux:
+			 c(z$qr$qraux, rep.int(0, d.p)),
+			 ## pivot:
+			 piv)
 	    }
 	} else { ## rank 0
 	    z <- list(coefficients = if (is.matrix(y)) matrix(NA,p,ncol(y))
@@ -156,7 +174,7 @@ lmrob <-
 		      residuals = y, scale = NA, fitted.values = 0 * y,
 		      cov = matrix(,0,0), rweights = rep.int(as.numeric(NA), NROW(y)),
 		      weights = w, rank = 0, df.residual = NROW(y),
-		      converged = TRUE, iter = 0)
+		      converged = TRUE, iter = 0, control=control)
 	    if (is.matrix(y)) colnames(z$coefficients) <- colnames(x)
 	    else names(z$coefficients) <- colnames(x)
 	    if(!is.null(offset)) z$residuals <- y - offset
@@ -166,6 +184,7 @@ lmrob <-
 	    z$fitted.values <- save.y - z$residuals
 	    z$weights <- w
 	    if (zero.weights) {
+                coef <- z$coefficients
 		coef[is.na(coef)] <- 0
 		f0 <- x0 %*% coef
 		if (ny > 1) {
@@ -183,9 +202,9 @@ lmrob <-
 		z$residuals <- save.r
 		z$fitted.values <- save.f
 		z$weights <- save.w
-		rweights <- z$rweights
+		rw <- z$rweights
 		z$rweights <- rep.int(0, length(save.w))
-		z$rweights[ok] <- rweights
+		z$rweights[ok] <- rw
 	    }
 	}
     }
@@ -212,6 +231,8 @@ lmrob <-
     z
 }
 
+if(getRversion() < "3.1.0") globalVariables(".lm.fit")
+
 ##' @title Warn about extraneous arguments in the "..."	 (of its caller)
 ##' @return
 ##' @author Martin Maechler, June 2012
@@ -311,9 +332,13 @@ print.summary.lmrob <-
 	      symbolic.cor = x$symbolic.cor,
 	      signif.stars = getOption("show.signif.stars"), ...)
 {
-    cat("\nCall:\n")
-    cat(paste(deparse(x$call), sep = "\n", collapse = "\n"),
-	"\n\n", sep = "")
+    cat("\nCall:\n",
+	cl <- paste(deparse(x$call, width.cutoff=72), sep = "\n", collapse = "\n"),
+	"\n", sep = "")
+    control <- lmrob.control.neededOnly(x$control)
+    ## if(!any(grepl("method *= *['\"]", cl)))## 'method = ".."' not explicitly visible above
+    cat(" \\--> method = \"", control$method, '"\n', sep = "")
+    ## else cat("\n")
     resid <- x$residuals
     df <- x$df
     rdf <- df[2L]
@@ -335,11 +360,11 @@ print.summary.lmrob <-
 		cat("\nExact fit detected\n\nCoefficients:\n")
 	    } else {
 		cat("\nAlgorithm did not converge\n")
-		if (x$control$method == "S")
+		if (control$method == "S")
 		    cat("\nCoefficients of the *initial* S-estimator:\n")
 		else
 		    cat(sprintf("\nCoefficients of the %s-estimator:\n",
-				x$control$method))
+				control$method))
 	    }
 	    printCoefmat(x$coef, digits = digits, signif.stars = signif.stars,
 			 ...)
@@ -379,62 +404,26 @@ print.summary.lmrob <-
 	}
 	cat("\n")
 
-	rweights <- x$rweights
-	if (!is.null(rweights)) {
-	    if (any(zero.weights <- x$weights == 0))
-		rweights <- rweights[!zero.weights]
-	    summarizeRobWeights(rweights, digits = digits, ...)
+	if (!is.null(rw <- x$rweights)) {
+	    if (any(zero.w <- x$weights == 0))
+		rw <- rw[!zero.w]
+	    summarizeRobWeights(rw, digits = digits, ...)
 	}
 
     } else cat("\nNo Coefficients\n")
 
-    ## modify control list to contain only parameters
-    ## that were actually used
-    control <- x$control
-    if (!is.null(control)) {
-	switch(sub("^(S|M-S).*", "\\1", control$method),
-	       S = { # remove all M-S specific control pars
-		   control$k.m_s <- NULL
-		   control$split.type <- NULL
-					# if large_n is not used, remove corresp control pars
-		   if (length(residuals) <= control$fast.s.large.n) {
-		       control$groups <- NULL
-		       control$n.group <- NULL
-		   }
-	       },
-	       `M-S` = { # remove all fast S specific control pars
-		   control$refine.tol <- NULL
-		   control$groups <- NULL
-		   control$n.group <- NULL
-		   control$best.r.s <- NULL
-		   control$k.fast.s <- NULL
-	       }, { # else: do not print any parameter used by initial ests. only
-		   control$tuning.chi <- NULL
-		   control$bb <- NULL
-		   control$refine.tol <- NULL
-		   control$nResample <- NULL
-		   control$groups <- NULL
-		   control$n.group <- NULL
-		   control$best.r.s <- NULL
-		   control$k.fast.s <- NULL
-		   control$k.max <- NULL
-		   control$k.m_s <- NULL
-		   control$split.type <- NULL
-		   control$mts <- NULL
-		   control$subsampling <- NULL
-	       } )
-	if (!grepl("D", control$method)) control$numpoints <- NULL
-	if (x$control$method == 'SM') control$method <- x$control$method <- 'MM'
-	printControl(control, digits = digits)
-    }
-
+    if (!is.null(control))
+	printControl(control, digits = digits, drop. = "method")
     invisible(x)
 }
 
 
 print.lmrob <- function(x, digits = max(3, getOption("digits") - 3), ...)
 {
-    cat("\nCall:\n", deparse(x$call), "\n\n", sep = "")
+    cat("\nCall:\n", cl <- deparse(x$call, width.cutoff=72), "\n", sep = "")
+    control <- lmrob.control.neededOnly(x$control)
+    if(!any(grepl("method *= *['\"]", cl)))## 'method = ".."' not explicitly visible above
+	cat(" \\--> method = \"", control$method, '"\n', sep = "") else cat("\n")
     if(length((cf <- coef(x)))) {
 	if( x$converged )
 	    cat("Coefficients:\n")
@@ -443,11 +432,11 @@ print.lmrob <- function(x, digits = max(3, getOption("digits") - 3), ...)
 		cat("Exact fit detected\n\nCoefficients:\n")
 	    } else {
 		cat("Algorithm did not converge\n\n")
-		if (x$control$method == "S")
+		if (control$method == "S")
 		    cat("Coefficients of the *initial* S-estimator:\n")
 		else
 		    cat(sprintf("Coefficients of the %s-estimator:\n",
-				x$control$method))
+				control$method))
 	    }
 	}
 	print(format(coef(x), digits = digits), print.gap = 2, quote = FALSE)
@@ -456,8 +445,24 @@ print.lmrob <- function(x, digits = max(3, getOption("digits") - 3), ...)
     invisible(x)
 }
 
-
-print.lmrob.S <- print.lmrob
+print.lmrob.S <- function(x, digits = max(3, getOption("digits") - 3), ...)
+{
+    cat("S-estimator lmrob.S():\n")
+    if(length((cf <- coef(x)))) {
+	if (x$converged)
+	    cat("Coefficients:\n")
+	else if (x$scale == 0)
+	    cat("Exact fit detected\n\nCoefficients:\n")
+	else
+	    cat("Algorithm did not converge\n\n")
+	print(format(cf, digits = digits), print.gap = 2, quote = FALSE)
+    } else cat("No coefficients\n")
+    cat("scale = ",format(x$scale, digits=digits), "; ",
+	if(x$converged)"converged" else "did NOT converge",
+	" in ", x$k.iter, " refinement steps\n")
+    printControl(x$control, digits = digits, drop. = "method")
+    invisible(x)
+}
 
 
 ## practically identical to  stats:::qr.lm :
@@ -475,6 +480,8 @@ summary.lmrob <- function(object, correlation = FALSE, symbolic.cor = FALSE, ...
 	stop("invalid 'lmrob' object:  no terms component")
     p <- object$rank
     df <- object$df.residual #was $degree.freedom
+    sigma <- object[["scale"]]
+    aliased <- is.na(coef(object))
     cf.nms <- c("Estimate", "Std. Error", "t value", "Pr(>|t|)")
     if (p > 0) {
 	n <- p + df
@@ -493,10 +500,8 @@ summary.lmrob <- function(object, correlation = FALSE, symbolic.cor = FALSE, ...
 	    if( ans$converged)
 		cbind(est, se, tval, 2 * pt(abs(tval), df, lower.tail = FALSE))
 	    else
-		cbind(est, if (object$scale == 0) 0 else NA, NA, NA)
+		cbind(est, if(sigma <= 0) 0 else NA, NA, NA)
 	dimnames(ans$coefficients) <- list(names(est), cf.nms)
-	ans$aliased <- is.na(coef(object)) # used in print method
-
 	ans$cov.unscaled <- object$cov
 	if(length(object$cov) > 1L)
 	    dimnames(ans$cov.unscaled) <- dimnames(ans$coefficients)[c(1,1)]
@@ -506,12 +511,13 @@ summary.lmrob <- function(object, correlation = FALSE, symbolic.cor = FALSE, ...
 	}
     } else { ## p = 0: "null model"
 	ans <- object
-	ans$aliased <- is.na(coef(object)) # used in print method
-	ans$df <- c(0L, df, length(ans$aliased))
+	ans$df <- c(0L, df, length(aliased))
 	ans$coefficients <- matrix(NA, 0L, 4L)
 	dimnames(ans$coefficients) <- list(NULL, cf.nms)
 	ans$cov.unscaled <- object$cov
     }
+    ans$aliased <- aliased # used in print method
+    ans$sigma <- sigma # 'sigma': in summary.lm() & 'fit.models' pkg
     class(ans) <- "summary.lmrob"
     ans
 }
@@ -553,7 +559,7 @@ weights.lmrob <- function(object, type = c("prior", "robustness"), ...) {
 
 printControl <-
     function(ctrl, digits = getOption("digits"),
-	     str.names = "seed",
+	     str.names = "seed", drop. = character(0),
 	     header = "Algorithmic parameters:",
 	     ...)
 {
@@ -564,14 +570,15 @@ printControl <-
 
     cat(header,"\n")
     is.str <- (nc <- names(ctrl)) %in% str.names
+    do. <- !is.str & !(nc %in% drop.)
     is.ch <- sapply(ctrl, is.character)
     real.ctrl <- sapply(ctrl, function(x)
 			length(x) > 0 && is.numeric(x) && x != round(x))
-    PR(ctrl[!is.str & real.ctrl], digits = digits, ...)
+    PR(ctrl[do. & real.ctrl], digits = digits, ...)
     ## non-real, non-char ones (typically integers), but dropping 0-length ones
-    PR(ctrl[!is.str & !is.ch & !real.ctrl], ...)
+    PR(ctrl[do. & !is.ch & !real.ctrl], ...)
     ## char ones
-    PR(ctrl[!is.str & is.ch], ...)
+    PR(ctrl[do. & is.ch], ...)
     if(any(is.str))
 	for(n in nc[is.str]) {
 	    cat(n,":")
@@ -602,16 +609,16 @@ summarizeRobWeights <-
 		formE <- function(e) formatC(e, digits = max(2, digits-3), width=1)
 		i0 <- which(w0)
 		maxw <- max(w[w0])
-		c3 <- paste("with |weight| ",
-			    if(maxw == 0) "= 0" else paste("<=", formE(maxw)),
-			    " ( < ", formE(eps), ");", sep='')
+		c3 <- paste0("with |weight| ",
+                             if(maxw == 0) "= 0" else paste("<=", formE(maxw)),
+			    " ( < ", formE(eps), ");")
 		cat0(if(n0 > 1) {
 		       cc <- sprintf("%d observations c(%s)",
 				     n0, strwrap(paste(i0, collapse=",")))
 		       c2 <- " are outliers"
-		       paste(cc,
+		       paste0(cc,
 			     if(nchar(cc)+ nchar(c2)+ nchar(c3) > getOption("width"))
-			     "\n	", c2, sep='')
+			     "\n	", c2)
 		     } else
 		       sprintf("observation %d is an outlier", i0),
 		     c3, "\n")
