@@ -20,7 +20,13 @@ stopifnot(all.equal(Y, fitted(fm1) + residuals(fm1), check.attr=FALSE),
 	  identical3(c(fitted(fm1)), predict(fm1), predict(fm1, newdata=DNase1)),
 	  ## robust fit :
 	  identical3(fitted(rm1), predict(rm1), predict(rm1, newdata=DNase1)),
-	  all.equal(Y, unname(fitted(rm1) + residuals(rm1)))
+	  all.equal(Y, unname(fitted(rm1) + residuals(rm1))),
+	  all.equal(coef(rm1),
+		    c(Asym=2.35963008, xmid=1.49945088, scal=1.04506392)),
+	  all.equal(sqrt(diag(rm1$cov)),
+                    ## 32b 0.08626872273,     0.0902194541,      0.03503833759
+		    c(Asym=0.0862687305, xmid=0.0902194608, scal=0.0350383389),
+                    tol = 7e-7)
 	  )
 
 ## From: "Pascal A. Niklaus" <pascal.niklaus@ieu.uzh.ch>
@@ -72,6 +78,24 @@ lines(x, predict(nlR1), col=4)
 legend("top", c("f(x)", "least squares", "robust"),
        col=c("tomato", palette()[3:4]), lty=c(2,1,1))
 
-## These both do *not* converge currently
-update(nlR1, psi = robustbase:::psi.bisquare)# failed in earlier versions
-update(nlR1, psi = robustbase:::psi.hampel)  # failed in earlier versions
+## These both now *do* converge, but failed earlier
+(nlbi <- update(nlR1, psi = robustbase:::psi.bisquare))
+(nlFH <- update(nlR1, psi = robustbase:::psi.hampel))
+lines(x, predict(nlbi), col=5)
+lines(x, predict(nlFH), col=6)
+
+stopifnot(nlR1$status == "converged", nlbi$status == "converged",
+	  nlFH$status == "converged",
+	  all.equal(coef(nlR1), c(a=9.914874,	b=3.98612416, c=0.250896252)),
+	  all.equal(coef(nlbi), c(a=9.94745828, b=3.95420997, c=0.253583549)),
+	  all.equal(coef(nlFH), c(a=9.94242831, b=3.97370746, c=0.252907618)),
+	  all.equal(1000*diag(vcov(nlR1)),
+		    c(a=16.167493, b=10.0986644, c=0.0200814189), tol = 7e-7),
+	  all.equal(1000*local({V <- vcov(nlFH); V[lower.tri(V, diag=TRUE)]}),
+		    c(11.454206, -6.82577421, 0.219594448,
+		      7.01186346, -0.282512742, 0.0139963737), tol = 7e-7),
+	  all.equal(predict(nlR1), predict(nlbi), tol = 0.05),
+	  all.equal(predict(nlR1), predict(nlFH), tol = 0.05),
+	  TRUE)
+
+cat('Time elapsed: ', proc.time(),'\n') # for ``statistical reasons''
